@@ -1,28 +1,36 @@
 /* VM library: native code patching for RISC-V .
 
    Copyright (C) 2017, 2019, 2020 Luca Saiu
+   Updated in 2022 by Luca Saiu
    Written by Luca Saiu
 
-   This file is part of Jitter.
+   This file is part of GNU Jitter.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include <stdint.h>
 #include <assert.h>
 #include <string.h>
 
+#if defined (JITTER_HAVE___RISCV_FLUSH_ICACHE)
+# include <sys/cachectl.h>  /* for __riscv_flush_icache */
+#else
+  /* Nothing serious: the generic GCC builtin __builtin___clear_cache should be
+     sufficient. */
+#endif
+
 #include <jitter/jitter-fatal.h>
 
 #include <jitter/jitter-arithmetic.h>
@@ -122,8 +130,11 @@ struct jitter_riscv_b_type_instruction
 void
 jitter_invalidate_icache (char *from, size_t byte_no)
 {
-  /* I am assuming that this does not need to do anything on RISC-V.  Not
-     tested on real hardware yet. */
+#if defined (JITTER_HAVE___RISCV_FLUSH_ICACHE)
+  __riscv_flush_icache (from, from + byte_no, 0);
+#else
+  /* Do nothing: __builtin___clear_cache is called elsewhere. */
+#endif
 }
 
 /* Return the distance in bytes, from the pointed instruction, to be patched,
