@@ -1,6 +1,6 @@
 /* pkl-anal.c - Analysis phases for the poke compiler.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -252,23 +252,6 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_comp_stmt)
 }
 PKL_PHASE_END_HANDLER
 
-/* Every node in the AST should have a valid location after parsing.
-   This handler is used in both anal1 and anal2.  */
-
-PKL_PHASE_BEGIN_HANDLER (pkl_anal_ps_default)
-{
-#if 0
-  if (!PKL_AST_LOC_VALID (PKL_AST_LOC (PKL_PASS_NODE)))
-    {
-      PKL_ICE (PKL_AST_NOLOC,
-               "node #%" PRIu64 " with code %d has no location",
-               PKL_AST_UID (PKL_PASS_NODE), PKL_AST_CODE (PKL_PASS_NODE));
-      PKL_PASS_ERROR;
-    }
-#endif
-}
-PKL_PHASE_END_HANDLER
-
 /* The arguments to a funcall should be either all named, or none
    named.  Also, it is not allowed to specify the same argument
    twice.  */
@@ -405,32 +388,21 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_type_function)
 }
 PKL_PHASE_END_HANDLER
 
-/* Make sure every BREAK statement has an associated entity.  */
+/* Make sure every BREAK and CONTINUE statement has an associated
+   entity.  */
 
-PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_break_stmt)
+PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_break_continue_stmt)
 {
-  pkl_ast_node break_stmt = PKL_PASS_NODE;
+  pkl_ast_node stmt = PKL_PASS_NODE;
 
-  if (!PKL_AST_BREAK_STMT_ENTITY (break_stmt))
+  if (!PKL_AST_BREAK_CONTINUE_STMT_ENTITY (stmt))
     {
-      PKL_ERROR (PKL_AST_LOC (break_stmt),
-                 "`break' statement without containing statement");
-      PKL_ANAL_PAYLOAD->errors++;
-      PKL_PASS_ERROR;
-    }
-}
-PKL_PHASE_END_HANDLER
+      int kind = PKL_AST_BREAK_CONTINUE_STMT_KIND (stmt);
 
-/* Make sure every CONTINUE statement has an associated entity.  */
-
-PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_continue_stmt)
-{
-  pkl_ast_node continue_stmt = PKL_PASS_NODE;
-
-  if (!PKL_AST_CONTINUE_STMT_ENTITY (continue_stmt))
-    {
-      PKL_ERROR (PKL_AST_LOC (continue_stmt),
-                 "`continue' statement without containing statement");
+      PKL_ERROR (PKL_AST_LOC (stmt),
+                 "%s statement without containing statement",
+                 kind == PKL_AST_BREAK_CONTINUE_STMT_KIND_BREAK
+                 ? "`break'" : "`continue'");
       PKL_ANAL_PAYLOAD->errors++;
       PKL_PASS_ERROR;
     }
@@ -699,6 +671,43 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_cons)
 }
 PKL_PHASE_END_HANDLER
 
+/* Some attributes are binary and thus require a second argument.
+   Make sure they have it.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_anal1_ps_op_attr)
+{
+  pkl_ast_node exp = PKL_PASS_NODE;
+  enum pkl_ast_attr attr = PKL_AST_EXP_ATTR (exp);
+
+  assert (attr != PKL_AST_ATTR_NONE);
+
+  switch (attr)
+    {
+    case PKL_AST_ATTR_ELEM:
+    case PKL_AST_ATTR_EOFFSET:
+    case PKL_AST_ATTR_ESIZE:
+    case PKL_AST_ATTR_ENAME:
+      if (PKL_AST_EXP_NUMOPS (exp) != 2)
+        {
+          PKL_ERROR (PKL_AST_LOC (exp),
+                     "attribute requires an argument");
+          PKL_ANAL_PAYLOAD->errors++;
+          PKL_PASS_ERROR;
+        }
+      break;
+    default:
+      if (PKL_AST_EXP_NUMOPS (exp) != 1)
+        {
+          PKL_ERROR (PKL_AST_LOC (exp),
+                     "attribute doesn't take any argument");
+          PKL_ANAL_PAYLOAD->errors++;
+          PKL_PASS_ERROR;
+        }
+      break;
+    }
+}
+PKL_PHASE_END_HANDLER
+
 struct pkl_phase pkl_phase_anal1 =
   {
    PKL_PHASE_PS_HANDLER (PKL_AST_SRC, pkl_anal_ps_src),
@@ -706,8 +715,7 @@ struct pkl_phase pkl_phase_anal1 =
    PKL_PHASE_PS_HANDLER (PKL_AST_PROGRAM, pkl_anal_ps_program),
    PKL_PHASE_PS_HANDLER (PKL_AST_STRUCT, pkl_anal1_ps_struct),
    PKL_PHASE_PS_HANDLER (PKL_AST_COMP_STMT, pkl_anal1_ps_comp_stmt),
-   PKL_PHASE_PS_HANDLER (PKL_AST_BREAK_STMT, pkl_anal1_ps_break_stmt),
-   PKL_PHASE_PS_HANDLER (PKL_AST_CONTINUE_STMT, pkl_anal1_ps_continue_stmt),
+   PKL_PHASE_PS_HANDLER (PKL_AST_BREAK_CONTINUE_STMT, pkl_anal1_ps_break_continue_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNCALL, pkl_anal1_ps_funcall),
    PKL_PHASE_PR_HANDLER (PKL_AST_FUNC, pkl_anal1_pr_func),
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNC, pkl_anal1_ps_func),
@@ -724,7 +732,7 @@ struct pkl_phase pkl_phase_anal1 =
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_OFFSET, pkl_anal1_ps_type_offset),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_ARRAY, pkl_anal1_ps_type_array),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_SL, pkl_anal1_ps_op_sl),
-   PKL_PHASE_PS_DEFAULT_HANDLER (pkl_anal_ps_default),
+   PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_ATTR, pkl_anal1_ps_op_attr),
   };
 
 
@@ -840,7 +848,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_funcall)
       && PKL_PASS_PARENT
       && PKL_AST_CODE (PKL_PASS_PARENT) != PKL_AST_EXP_STMT)
     {
-      /* Note that this error is catched earlier in
+      /* Note that this error is caught earlier in
          pkl_typify1_ps_funcall */
       PKL_ERROR (PKL_AST_LOC (funcall_function),
                  "call to void function in expression");
@@ -898,14 +906,14 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_type_struct)
 
   for (t = struct_type_elems; t; t = PKL_AST_CHAIN (t))
     {
-      /* Process only struct type fields.  */
-      if (PKL_AST_CODE (t) == PKL_AST_STRUCT_TYPE_FIELD)
+      /* Process only struct type fields that are not computed.  */
+      if (PKL_AST_CODE (t) == PKL_AST_STRUCT_TYPE_FIELD
+          && !PKL_AST_STRUCT_TYPE_FIELD_COMPUTED_P (t))
         {
           pkl_ast_node constraint = PKL_AST_STRUCT_TYPE_FIELD_CONSTRAINT (t);
-          pkl_ast_node optcond = PKL_AST_STRUCT_TYPE_FIELD_OPTCOND (t);
           pkl_ast_node elem_type = PKL_AST_STRUCT_TYPE_FIELD_TYPE (t);
 
-          if (optcond)
+          if (PKL_AST_STRUCT_TYPE_FIELD_OPTIONAL_P (t))
             {
               PKL_ERROR (PKL_AST_LOC (t),
                          "optional fields are not allowed in unions");
@@ -958,6 +966,43 @@ PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_type_struct)
 }
 PKL_PHASE_END_HANDLER
 
+/* The template in an asm statement shall be a constant string.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_asm_stmt)
+{
+  pkl_ast_node ass_stmt = PKL_PASS_NODE;
+  pkl_ast_node ass_stmt_template = PKL_AST_ASM_STMT_TEMPLATE (ass_stmt);
+
+  if (!ass_stmt_template
+      || PKL_AST_CODE (ass_stmt_template) != PKL_AST_STRING)
+    {
+      PKL_ERROR (PKL_AST_LOC (ass_stmt_template),
+                 "template argument to asm must be constant");
+      PKL_ANAL_PAYLOAD->errors ++;
+      PKL_PASS_ERROR;
+    }
+}
+PKL_PHASE_END_HANDLER
+
+/* The template in an asm expression shall be a constant string.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_anal2_ps_asm_exp)
+{
+  pkl_ast_node ass_exp = PKL_PASS_NODE;
+  pkl_ast_node ass_exp_template = PKL_AST_ASM_EXP_TEMPLATE (ass_exp);
+
+  if (!ass_exp_template
+      || PKL_AST_CODE (ass_exp_template) != PKL_AST_STRING)
+    {
+      PKL_ERROR (PKL_AST_LOC (ass_exp_template),
+                 "template argument to asm must be constant");
+      PKL_ANAL_PAYLOAD->errors ++;
+      PKL_PASS_ERROR;
+    }
+}
+PKL_PHASE_END_HANDLER
+
+
 struct pkl_phase pkl_phase_anal2 =
   {
    PKL_PHASE_PS_HANDLER (PKL_AST_SRC, pkl_anal_ps_src),
@@ -970,8 +1015,9 @@ struct pkl_phase pkl_phase_anal2 =
    PKL_PHASE_PS_HANDLER (PKL_AST_RETURN_STMT, pkl_anal2_ps_return_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNCALL, pkl_anal2_ps_funcall),
    PKL_PHASE_PS_HANDLER (PKL_AST_STRUCT_TYPE_FIELD, pkl_anal2_ps_struct_type_field),
+   PKL_PHASE_PS_HANDLER (PKL_AST_ASM_STMT, pkl_anal2_ps_asm_stmt),
+   PKL_PHASE_PS_HANDLER (PKL_AST_ASM_EXP, pkl_anal2_ps_asm_exp),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_STRUCT, pkl_anal2_ps_type_struct),
-   PKL_PHASE_PS_DEFAULT_HANDLER (pkl_anal_ps_default),
   };
 
 
