@@ -1,6 +1,6 @@
 /* pkl-fold.c - Constant folding phase for the poke compiler. */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -616,7 +616,9 @@ EMUL_UU (bnoto) { return ~op; }
                   break;                                                \
                 case PKL_AST_OP_POW:                                    \
                   /* XXX constant-folding this implies to not use */    \
-                  /* ipow, but a version that checks overflow.  */      \
+                  /* ipow, but a version that checks overflow.  Please */ \
+                  /* remove the workaround in pkl_trans2_ps_exp */      \
+                  /* once this gets fixed.  */                          \
                   PKL_PASS_DONE;                                        \
                   break;                                                \
                 default:                                                \
@@ -760,7 +762,7 @@ EMUL_UU (bnoto) { return ~op; }
                                                                               \
           bound = pkl_ast_make_binary_exp (PKL_PASS_AST, PKL_AST_OP_ADD,      \
                                            bound1, bound2);                   \
-          PKL_AST_TYPE (bound) = btype1;                                      \
+          PKL_AST_TYPE (bound) = ASTREF (btype1);                             \
           PKL_AST_TYPE (new) = ASTREF (pkl_ast_make_array_type (              \
               PKL_PASS_AST, PKL_AST_TYPE_A_ETYPE (op1_type), bound));         \
           PKL_AST_TYPE_COMPLETE (PKL_AST_TYPE (new))                          \
@@ -1130,7 +1132,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_fold_ps_cast)
            PKL_AST_INTEGER_VALUE (unit));
 
       /* Calculate the new unit.  It should always be a new node,
-         since otherwise we may be chaning an integer node that is
+         since otherwise we may be chaining an integer node that is
          also part of an unit declaration, or who knows what.  */
       {
         pkl_ast_node unit_type = PKL_AST_TYPE (unit);
