@@ -1,6 +1,6 @@
 /* pkl-promo.c - Operand promotion phase for the poke compiler.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 /* This file implements a compiler phase that promotes the operands of
    expressions following language rules.  This phase expects that
-   every expression operand is anotated with its proper type.  */
+   every expression operand is annotated with its proper type.  */
 
 #include <config.h>
 
@@ -592,6 +592,7 @@ PKL_PHASE_END_HANDLER
            OFFSET   x OFFSET   -> BOOL
            ARRAY    x ARRAY    -> BOOL
            STRUCT   x STRUCT   -> BOOL
+           FUNCTION x FUNCTION -> BOOL
 
    In the I x I -> I configuration, the types of the operands are
    promoted in a way both operands end having the same type, following
@@ -600,8 +601,8 @@ PKL_PHASE_END_HANDLER
    The same logic is applied to the magnitudes of the offset operands
    in the O x O -> O configuration.
 
-   No operand promotion is performed in the S x S -> S
-   configuration.  Ditto for A x A -> A.  */
+   No operand promotion is performed in the S x S -> B
+   configuration.  Ditto for A x A -> B and F x F -> B.  */
 
 PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_rela)
 {
@@ -683,6 +684,9 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_rela)
     case PKL_TYPE_STRUCT:
       /* Nothing to do.  */
       break;
+    case PKL_TYPE_FUNCTION:
+      /* Nothing to do.  */
+      break;
     default:
       goto error;
     }
@@ -829,22 +833,43 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_unary)
 }
 PKL_PHASE_END_HANDLER
 
-/* Handler for promoting indexes in indexers to unsigned 64 bit
-   values.  */
+/* Handler for promoting integral indexes in indexers:
+
+   - Integral indexes are promoted to uint<64>.
+   - Offset indexes are promoted to offset<uint<64>,1>.  */
 
 PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_indexer)
 {
   int restart;
   pkl_ast_node node = PKL_PASS_NODE;
+  pkl_ast_node index = PKL_AST_INDEXER_INDEX (node);
+  pkl_ast_node index_type = PKL_AST_TYPE (index);
 
-  if (!promote_integral (PKL_PASS_AST, 64, 0,
-                         &PKL_AST_INDEXER_INDEX (node), &restart))
+  if (PKL_AST_TYPE_CODE (index_type) == PKL_TYPE_INTEGRAL
+      && !promote_integral (PKL_PASS_AST, 64, 0,
+                            &PKL_AST_INDEXER_INDEX (node), &restart))
     {
       PKL_ICE (PKL_AST_LOC (node),
-               "couldn't promote indexer subscript");
+               "couldn't promote integral indexer subscript");
       PKL_PASS_ERROR;
     }
 
+  if (PKL_AST_TYPE_CODE (index_type) == PKL_TYPE_OFFSET)
+    {
+      pkl_ast_node unit_bit = pkl_ast_make_integer (PKL_PASS_AST, 1);
+      unit_bit = ASTREF (unit_bit);
+
+      if (!promote_offset (PKL_PASS_AST, 64, 0, unit_bit,
+                           &PKL_AST_INDEXER_INDEX (node), &restart))
+        {
+          PKL_ICE (PKL_AST_LOC (node),
+                   "couldn't promote offset indexer subscript");
+          PKL_PASS_ERROR;
+        }
+
+      pkl_ast_node_free (unit_bit);
+    }
+
   PKL_PASS_RESTART = restart;
 }
 PKL_PHASE_END_HANDLER
@@ -937,7 +962,6 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_type_array)
         pkl_ast_node unit_bit = pkl_ast_make_integer (PKL_PASS_AST, 1);
 
         unit_bit = ASTREF (unit_bit);
-        PKL_AST_LOC (unit_bit) = PKL_AST_LOC (PKL_PASS_NODE);
 
         if (!promote_offset (PKL_PASS_AST,
                              64, 0, unit_bit,
@@ -984,7 +1008,6 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_array_initializer)
           index_type = pkl_ast_make_integral_type (PKL_PASS_AST,
                                                    64, 0);
           PKL_AST_TYPE (index) = ASTREF (index_type);
-          PKL_AST_LOC (index_type) = PKL_AST_LOC (node);
           PKL_PASS_RESTART = 1;
         }
     }
@@ -1002,6 +1025,10 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_ass_stmt)
   pkl_ast_node lvalue_type = PKL_AST_TYPE (lvalue);
   int restart = 0;
 
+  if (!exp)
+    /* Nothing to promote.  */
+    PKL_PASS_DONE;
+
   /* At this point it is assured that exp_type is promoteable to
      lvalue_type, or typify1 wouldn't have allowed this node to
      pass.  */
@@ -1183,21 +1210,21 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_return_stmt)
 }
 PKL_PHASE_END_HANDLER
 
-/* Promote arguments to printf.  */
+/* Promote arguments to format.  */
 
-PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_print_stmt)
+PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_format)
 {
-  pkl_ast_node print_stmt = PKL_PASS_NODE;
-  pkl_ast_node print_stmt_types = PKL_AST_PRINT_STMT_TYPES (print_stmt);
-  pkl_ast_node print_stmt_args = PKL_AST_PRINT_STMT_ARGS (print_stmt);
+  pkl_ast_node format = PKL_PASS_NODE;
+  pkl_ast_node format_types = PKL_AST_FORMAT_TYPES (format);
+  pkl_ast_node format_args = PKL_AST_FORMAT_ARGS (format);
 
   pkl_ast_node type, arg;
 
-  for (arg = print_stmt_args, type = print_stmt_types;
+  for (arg = format_args, type = format_types;
        arg && type;
        arg = PKL_AST_CHAIN (arg), type = PKL_AST_CHAIN (type))
     {
-      pkl_ast_node arg_exp = PKL_AST_PRINT_STMT_ARG_EXP (arg);
+      pkl_ast_node arg_exp = PKL_AST_FORMAT_ARG_EXP (arg);
 
       /* Skip arguments without associated values.  Also skip
          arguments with declared type ANY (%v) */
@@ -1207,11 +1234,11 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_print_stmt)
           int restart;
 
           if (!promote_node (PKL_PASS_AST,
-                             &PKL_AST_PRINT_STMT_ARG_EXP (arg),
+                             &PKL_AST_FORMAT_ARG_EXP (arg),
                              type, &restart))
             {
               PKL_ICE (PKL_AST_LOC (arg),
-                       "couldn't promote printf argument initializer");
+                       "couldn't promote format argument initializer");
               PKL_PASS_ERROR;
             }
 
@@ -1271,7 +1298,6 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_map)
   pkl_ast_node unit_bit = pkl_ast_make_integer (PKL_PASS_AST, 1);
 
   unit_bit = ASTREF (unit_bit);
-  PKL_AST_LOC (unit_bit) = PKL_AST_LOC (PKL_PASS_NODE);
 
   if (!promote_offset (PKL_PASS_AST,
                        64, 0, unit_bit,
@@ -1351,7 +1377,8 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_struct_type_field)
   pkl_ast_node elem_type = PKL_AST_STRUCT_TYPE_FIELD_TYPE (elem);
   pkl_ast_node elem_constraint = PKL_AST_STRUCT_TYPE_FIELD_CONSTRAINT (elem);
   pkl_ast_node elem_initializer = PKL_AST_STRUCT_TYPE_FIELD_INITIALIZER (elem);
-  pkl_ast_node elem_optcond = PKL_AST_STRUCT_TYPE_FIELD_OPTCOND (elem);
+  pkl_ast_node elem_optcond_pre = PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_PRE (elem);
+  pkl_ast_node elem_optcond_post = PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_POST (elem);
   pkl_ast_node elem_label = PKL_AST_STRUCT_TYPE_FIELD_LABEL (elem);
 
   if (elem_constraint)
@@ -1400,9 +1427,9 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_struct_type_field)
       PKL_PASS_RESTART = PKL_PASS_RESTART || restart;
     }
 
-  if (elem_optcond)
+  if (elem_optcond_pre)
     {
-      pkl_ast_node optcond_type = PKL_AST_TYPE (elem_optcond);
+      pkl_ast_node optcond_type = PKL_AST_TYPE (elem_optcond_pre);
       int restart = 0;
 
       /* Handle integral structure.  */
@@ -1412,17 +1439,46 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_struct_type_field)
 
       if (PKL_AST_TYPE_CODE (optcond_type) != PKL_TYPE_INTEGRAL)
         {
-          PKL_ICE (PKL_AST_LOC (elem_optcond),
-                   "non-promoteable struct field optcond at promo time");
+          PKL_ICE (PKL_AST_LOC (elem_optcond_pre),
+                   "non-promoteable struct field pre optcond at promo time");
           PKL_PASS_ERROR;
         }
 
       if (!promote_integral (PKL_PASS_AST, 32, 1,
-                             &PKL_AST_STRUCT_TYPE_FIELD_OPTCOND (elem),
+                             &PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_PRE (elem),
                              &restart))
         {
-          PKL_ICE (PKL_AST_LOC (elem_optcond),
-                   "couldn't promote struct field optcond");
+          PKL_ICE (PKL_AST_LOC (elem_optcond_pre),
+                   "couldn't promote struct field pre optcond");
+          PKL_PASS_ERROR;
+        }
+
+      PKL_PASS_RESTART = PKL_PASS_RESTART || restart;
+    }
+
+  if (elem_optcond_post)
+    {
+      pkl_ast_node optcond_type = PKL_AST_TYPE (elem_optcond_post);
+      int restart = 0;
+
+      /* Handle integral structure.  */
+      if (PKL_AST_TYPE_CODE (optcond_type) == PKL_TYPE_STRUCT
+          && PKL_AST_TYPE_S_ITYPE (optcond_type))
+        optcond_type = PKL_AST_TYPE_S_ITYPE (optcond_type);
+
+      if (PKL_AST_TYPE_CODE (optcond_type) != PKL_TYPE_INTEGRAL)
+        {
+          PKL_ICE (PKL_AST_LOC (elem_optcond_post),
+                   "non-promoteable struct field post optcond at promo time");
+          PKL_PASS_ERROR;
+        }
+
+      if (!promote_integral (PKL_PASS_AST, 32, 1,
+                             &PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_POST (elem),
+                             &restart))
+        {
+          PKL_ICE (PKL_AST_LOC (elem_optcond_post),
+                   "couldn't promote struct field post optcond");
           PKL_PASS_ERROR;
         }
 
@@ -1487,6 +1543,42 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_in)
 }
 PKL_PHASE_END_HANDLER
 
+/* Promote the arguments of some attributes.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_attr)
+{
+  pkl_ast_node exp = PKL_PASS_NODE;
+  enum pkl_ast_attr attr = PKL_AST_EXP_ATTR (exp);
+  int restart = 0;
+
+  switch (attr)
+    {
+    case PKL_AST_ATTR_ELEM:
+    case PKL_AST_ATTR_EOFFSET:
+    case PKL_AST_ATTR_ESIZE:
+    case PKL_AST_ATTR_ENAME:
+
+      /* The argument of the attribute must be promoted to an unsigned
+         64-bit integer.  */
+      if (!promote_integral (PKL_PASS_AST,
+                             64, 0,
+                             &PKL_AST_EXP_OPERAND (exp, 1),
+                             &restart))
+        {
+          PKL_ICE (PKL_AST_LOC (exp),
+                   "couldn't pormote argument of attribute expression #%" PRIu64,
+                   PKL_AST_UID (exp));
+          PKL_PASS_ERROR;
+        }
+
+      PKL_PASS_RESTART = restart;
+      break;
+    default:
+      break;
+    }
+}
+PKL_PHASE_END_HANDLER
+
 PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_cons)
 {
   pkl_ast_node cons = PKL_PASS_NODE;
@@ -1582,6 +1674,67 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_cons)
 }
 PKL_PHASE_END_HANDLER
 
+/* When an integral value is casted to an integral struct, it shall be
+   promoted to the later integral type.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_cast)
+{
+  pkl_ast_node cast = PKL_PASS_NODE;
+  pkl_ast_node exp = PKL_AST_CAST_EXP (cast);
+  pkl_ast_node from_type = PKL_AST_TYPE (exp);
+  pkl_ast_node to_type = PKL_AST_CAST_TYPE (cast);
+
+  if (PKL_AST_TYPE_CODE (to_type) == PKL_TYPE_STRUCT)
+    {
+      pkl_ast_node itype = PKL_AST_TYPE_S_ITYPE (to_type);
+
+      if (itype
+          && PKL_AST_TYPE_CODE (from_type) == PKL_TYPE_INTEGRAL)
+        {
+          int restart = 0;
+
+          if (!promote_integral (PKL_PASS_AST,
+                                 PKL_AST_TYPE_I_SIZE (itype),
+                                 PKL_AST_TYPE_I_SIGNED_P (itype),
+                                 &PKL_AST_CAST_EXP (cast),
+                                 &restart))
+            {
+              PKL_ICE (PKL_AST_LOC (exp),
+                       "couldn't promote integral to integral struct");
+              PKL_PASS_ERROR;
+            }
+
+          PKL_PASS_RESTART = restart;
+        }
+    }
+}
+PKL_PHASE_END_HANDLER
+
+/* The second operand of an APUSH operation must be promoted to the
+   type of the elements of the first operator.  */
+
+PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_apush)
+{
+  pkl_ast_node exp = PKL_PASS_NODE;
+  pkl_ast_node op1 = PKL_AST_EXP_OPERAND (exp, 0);
+  pkl_ast_node op1_type = PKL_AST_TYPE (op1);
+  pkl_ast_node op1_elem_type = PKL_AST_TYPE_A_ETYPE (op1_type);
+  int restart = 0;
+
+  if (!promote_node (PKL_PASS_AST,
+                     &PKL_AST_EXP_OPERAND (PKL_PASS_NODE, 1),
+                     op1_elem_type,
+                     &restart))
+    {
+      PKL_ICE (PKL_AST_LOC (PKL_AST_EXP_OPERAND (PKL_PASS_NODE, 1)),
+               "couldn't promote argument to apush");
+      PKL_PASS_ERROR;
+    }
+
+  PKL_PASS_RESTART = restart;
+}
+PKL_PHASE_END_HANDLER
+
 struct pkl_phase pkl_phase_promo =
   {
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_EQ, pkl_promo_ps_op_rela),
@@ -1597,6 +1750,7 @@ struct pkl_phase pkl_phase_promo =
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_BAND, pkl_promo_ps_op_binary_intoffstrarr),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_AND, pkl_promo_ps_op_binary),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_OR, pkl_promo_ps_op_binary),
+   PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_IMPL, pkl_promo_ps_op_binary),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_NOT, pkl_promo_ps_op_unary),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_NEG, pkl_promo_ps_op_unary),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_POS, pkl_promo_ps_op_unary),
@@ -1610,6 +1764,8 @@ struct pkl_phase pkl_phase_promo =
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_DIV, pkl_promo_ps_op_div),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_CEILDIV, pkl_promo_ps_op_div),
    PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_IN, pkl_promo_ps_op_in),
+   PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_ATTR, pkl_promo_ps_op_attr),
+   PKL_PHASE_PS_OP_HANDLER (PKL_AST_OP_APUSH, pkl_promo_ps_op_apush),
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNC_ARG, pkl_promo_ps_func_arg),
    PKL_PHASE_PS_HANDLER (PKL_AST_MAP, pkl_promo_ps_map),
    PKL_PHASE_PS_HANDLER (PKL_AST_INDEXER, pkl_promo_ps_indexer),
@@ -1618,12 +1774,13 @@ struct pkl_phase pkl_phase_promo =
    PKL_PHASE_PS_HANDLER (PKL_AST_FUNCALL, pkl_promo_ps_funcall),
    PKL_PHASE_PS_HANDLER (PKL_AST_ASS_STMT, pkl_promo_ps_ass_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_RETURN_STMT, pkl_promo_ps_return_stmt),
-   PKL_PHASE_PS_HANDLER (PKL_AST_PRINT_STMT, pkl_promo_ps_print_stmt),
+   PKL_PHASE_PS_HANDLER (PKL_AST_FORMAT, pkl_promo_ps_format),
    PKL_PHASE_PS_HANDLER (PKL_AST_IF_STMT, pkl_promo_ps_if_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_LOOP_STMT, pkl_promo_ps_loop_stmt),
    PKL_PHASE_PS_HANDLER (PKL_AST_STRUCT_TYPE_FIELD, pkl_promo_ps_struct_type_field),
    PKL_PHASE_PS_HANDLER (PKL_AST_COND_EXP, pkl_promo_ps_cond_exp),
    PKL_PHASE_PS_HANDLER (PKL_AST_CONS, pkl_promo_ps_cons),
+   PKL_PHASE_PS_HANDLER (PKL_AST_CAST, pkl_promo_ps_cast),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_ARRAY, pkl_promo_ps_type_array),
    PKL_PHASE_PS_TYPE_HANDLER (PKL_TYPE_OFFSET, pkl_promo_ps_type_offset),
   };
