@@ -1,6 +1,6 @@
 /* pvm-val.c - Values for the PVM.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,6 @@
 
 static pvm_val string_type;
 static pvm_val void_type;
-static pvm_val any_type;
 
 /* We are currently only supporting a relatively small number of
    integral types, i.e. signed and unsigned types of sizes 1 to 64
@@ -253,7 +252,7 @@ pvm_array_set (pvm_val arr, pvm_val idx, pvm_val val)
   /* Update the element with the given value.  */
   PVM_VAL_ARR_ELEM_VALUE (arr, index) = val;
 
-  /* Recalculate the bit-offset of all the elemens following the
+  /* Recalculate the bit-offset of all the elements following the
      element just updated.  */
   for (i = index + 1; i < nelem; ++i)
     {
@@ -372,6 +371,28 @@ pvm_ref_struct_cstr (pvm_val sct, const char *name)
   return PVM_NULL;
 }
 
+void
+pvm_ref_set_struct_cstr (pvm_val sct, const char *fname,
+                         pvm_val value)
+{
+  size_t nfields, i;
+  struct pvm_struct_field *fields;
+
+  assert (PVM_IS_SCT (sct));
+
+  nfields = PVM_VAL_ULONG (PVM_VAL_SCT_NFIELDS (sct));
+  fields = PVM_VAL_SCT (sct)->fields;
+
+  for (i = 0; i < nfields; ++i)
+    {
+      if (!PVM_VAL_SCT_FIELD_ABSENT_P (sct, i)
+          && fields[i].name != PVM_NULL
+          && STREQ (PVM_VAL_STR (fields[i].name),
+                    fname))
+        fields[i].value = value;
+    }
+}
+
 pvm_val
 pvm_ref_struct (pvm_val sct, pvm_val name)
 {
@@ -495,12 +516,6 @@ pvm_make_void_type (void)
   return void_type;
 }
 
-pvm_val
-pvm_make_any_type (void)
-{
-  return any_type;
-}
-
 pvm_val
 pvm_make_offset_type (pvm_val base_type, pvm_val unit)
 {
@@ -512,12 +527,14 @@ pvm_make_offset_type (pvm_val base_type, pvm_val unit)
 }
 
 pvm_val
-pvm_make_array_type (pvm_val type, pvm_val bound)
+pvm_make_array_type (pvm_val type, pvm_val bounder)
 {
   pvm_val atype = pvm_make_type (PVM_TYPE_ARRAY);
 
+  assert (PVM_IS_CLS (bounder));
+
   PVM_VAL_TYP_A_ETYPE (atype) = type;
-  PVM_VAL_TYP_A_BOUND (atype) = bound;
+  PVM_VAL_TYP_A_BOUND (atype) = bounder;
   return atype;
 }
 
@@ -1143,7 +1160,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, ulongval, size, 1);
+          pk_print_binary (pk_puts, ulongval, size, /*signed_p*/ 1,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1172,7 +1190,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, (uint64_t) uintval, size, 1);
+          pk_print_binary (pk_puts, (uint64_t) uintval, size, /*signed*/ 1,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1201,7 +1220,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, ulongval, size, 0);
+          pk_print_binary (pk_puts, ulongval, size, /*signed_p*/ 0,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1223,7 +1243,8 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
       if (base == 2)
         {
           pk_puts ("0b");
-          pk_print_binary (pk_puts, uintval, size, 0);
+          pk_print_binary (pk_puts, uintval, size, /*signed_p*/ 0,
+                           /*use_suffix_p*/ 1);
         }
       else
         {
@@ -1419,15 +1440,15 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
                   pk_puts ("=");
                 }
               PVM_PRINT_VAL_1 (value, ndepth + 1);
-            }
 
-          if (maps && offset != PVM_NULL)
-            {
-              pk_puts (" @ ");
-              pk_term_class ("offset");
-              PVM_PRINT_VAL_1 (offset, ndepth);
-              pk_puts ("#b");
-              pk_term_end_class ("offset");
+              if (maps && offset != PVM_NULL)
+                {
+                  pk_puts (" @ ");
+                  pk_term_class ("offset");
+                  PVM_PRINT_VAL_1 (offset, ndepth);
+                  pk_puts ("#b");
+                  pk_term_end_class ("offset");
+                }
             }
         }
 
@@ -1475,11 +1496,6 @@ pvm_print_val_1 (pvm vm, int depth, int mode, int base, int indent,
         case PVM_TYPE_VOID:
           pk_puts ("void");
           break;
-        case PVM_TYPE_ANY:
-          pk_term_class ("any");
-          pk_puts ("any");
-          pk_term_end_class ("any");
-          break;
         case PVM_TYPE_ARRAY:
           PVM_PRINT_VAL_1 (PVM_VAL_TYP_A_ETYPE (val), ndepth);
           pk_puts ("[");
@@ -1629,6 +1645,8 @@ pvm_typeof (pvm_val val)
     type = PVM_VAL_SCT_TYPE (val);
   else if (PVM_IS_TYP (val))
     type = val;
+  else if (PVM_IS_CLS (val))
+    type = PVM_NULL;
   else
     assert (0);
 
@@ -1656,7 +1674,6 @@ pvm_type_equal_p (pvm_val type1, pvm_val type2)
         return (t1_size == t2_size && t1_signed == t2_signed);
       }
     case PVM_TYPE_STRING:
-    case PVM_TYPE_ANY:
     case PVM_TYPE_VOID:
       return 1;
     case PVM_TYPE_ARRAY:
@@ -1797,12 +1814,10 @@ pvm_val_initialize (void)
 
   pvm_alloc_add_gc_roots (&string_type, 1);
   pvm_alloc_add_gc_roots (&void_type, 1);
-  pvm_alloc_add_gc_roots (&any_type, 1);
   pvm_alloc_add_gc_roots (&common_int_types, 65 * 2);
 
   string_type = pvm_make_type (PVM_TYPE_STRING);
   void_type = pvm_make_type (PVM_TYPE_VOID);
-  any_type = pvm_make_type (PVM_TYPE_ANY);
 
   for (i = 0; i < 65; ++i)
     for (j = 0; j < 2; ++j)
@@ -1814,6 +1829,5 @@ pvm_val_finalize (void)
 {
   pvm_alloc_remove_gc_roots (&string_type, 1);
   pvm_alloc_remove_gc_roots (&void_type, 1);
-  pvm_alloc_remove_gc_roots (&any_type, 1);
   pvm_alloc_remove_gc_roots (&common_int_types, 65 * 2);
 }
