@@ -1,6 +1,6 @@
 /* pvm.h - PVM values  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -282,7 +282,7 @@ typedef struct pvm_array *pvm_array;
    how to obtain these values.
 
    OFFSET is an ulong<64> value holding the bit offset of the element,
-   relative to the begginnig of the IO space.  If the array is not
+   relative to the beginnig of the IO space.  If the array is not
    mapped then this is PVM_NULL.
 
    OFFSET_BACK is a backup area used by the reloc instructions.
@@ -446,7 +446,6 @@ enum pvm_type_code
   PVM_TYPE_STRUCT,
   PVM_TYPE_OFFSET,
   PVM_TYPE_CLOSURE,
-  PVM_TYPE_ANY,
   PVM_TYPE_VOID
 };
 
