@@ -1,6 +1,6 @@
 /* poke.c - Interactive editor for binary files.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@
 #include "pk-cmd.h"
 #include "pk-repl.h"
 #include "pk-utils.h"
-#include "pk-mi.h"
 #include "pk-map.h"
 #include "pk-ios.h"
 
@@ -51,29 +50,22 @@ int poke_interactive_p;
 int poke_hserver_p;
 #endif
 
-/* The following global indicates whether the MI shall be used.  */
-int poke_mi_p;
-
-#ifdef POKE_MI
-/* This build of poke supports mi if the user wants it.  */
-static const int mi_supported_p = 1;
-#else
-/* This build of poke does not support mi.  */
-static const int mi_supported_p = 0;
-int pk_mi (void) {assert (0); return 0;}
-#endif
-
 /* The following global indicates whether poke should be as terse as
    possible in its output.  This is useful when running poke from
    other programs.  */
 
 int poke_quiet_p;
 
-/* The following global contains the directory holding the program's
+/* The following global contains the directory holding libpoke's
    architecture independent files, such as scripts.  */
 
 char *poke_datadir;
 
+/* The following global contains the directory holding the program's
+   architecture independent files, such as scripts.  */
+
+char *poke_appdir;
+
 /* The following global contains the directory holding the program's
    info files.  */
 
@@ -104,7 +96,7 @@ char *poke_mapsdir;
 char *poke_docdir;
 
 /* The following global determines whether auto-maps shall be
-   acknowleged when loading files by default.  Defaults to 1, but can
+   acknowledged when loading files by default.  Defaults to 1, but can
    be changed to 0 using the --no-auto-map command-line option.  */
 
 int poke_default_auto_map_p = 1;
@@ -144,7 +136,6 @@ enum
   STYLE_ARG,
   STYLE_DARK_ARG,
   STYLE_BRIGHT_ARG,
-  MI_ARG,
   NO_AUTO_MAP_ARG,
   NO_HSERVER_ARG
 };
@@ -162,12 +153,14 @@ static const struct option long_options[] =
   {"style", required_argument, NULL, STYLE_ARG},
   {"style-dark", no_argument, NULL, STYLE_DARK_ARG},
   {"style-bright", no_argument, NULL, STYLE_BRIGHT_ARG},
-  {"mi", no_argument, NULL, MI_ARG},
   {"no-auto-map", no_argument, NULL, NO_AUTO_MAP_ARG},
   {"no-hserver", no_argument, NULL, NO_HSERVER_ARG},
   {NULL, 0, NULL, 0},
 };
 
+/* String passed to getopt_long.  */
+static const char *const poke_getopt_string = "ql:c:s:L:";
+
 static void
 print_help (void)
 {
@@ -200,10 +193,6 @@ print_help (void)
   puts (_("      --style-dark                    use default style for dark backgrounds"));
   puts (_("      --style-bright                  use default style for bright backgrounds"));
 
-  puts ("");
-  puts (_("Machine interface:"));
-  puts (_("      --mi                            use the MI in stdin/stdout"));
-
   puts ("");
   /* TRANSLATORS: --help output, less used GNU poke arguments.
      no-wrap */
@@ -336,18 +325,25 @@ set_script_args (int argc, char *argv[])
   int i, nargs;
   uint64_t index;
   pk_val argv_array;
+  pk_val array_type_bounder = pk_decl_val (poke_compiler, "pk_bounder_unbounded");
 
   /* Look for -L SCRIPT */
   for (i = 0; i < argc; ++i)
-    if (STREQ (argv[i], "-L"))
-      break;
+    {
+      char *optarg = argv[i];
+
+      if (strlen (optarg) >= 2
+          && optarg[0] == '-'
+          && optarg[1] == 'L')
+        break;
+    }
 
   /* Any argument after SCRIPT is an argument for the script.  */
   i = i + 2;
   nargs = argc - i;
   argv_array = pk_make_array (pk_make_uint (nargs, 64),
                               pk_make_array_type (pk_make_string_type (),
-                                                  PK_NULL /* bound */));
+                                                  array_type_bounder));
 
   for (index = 0; i < argc; ++i, ++index)
     pk_array_insert_elem (argv_array, index,
@@ -364,7 +360,7 @@ parse_args_1 (int argc, char *argv[])
 
   while ((ret = getopt_long (argc,
                              argv,
-                             "ql:c:s:L:",
+                             poke_getopt_string,
                              long_options,
                              NULL)) != -1)
     {
@@ -379,18 +375,6 @@ parse_args_1 (int argc, char *argv[])
           pk_print_version (0 /* hand_p */);
           exit (EXIT_SUCCESS);
           break;
-        case MI_ARG:
-          if (!mi_supported_p)
-            {
-              fputs (_("MI is not built into this instance of poke\n"),
-                     stderr);
-              exit (EXIT_FAILURE);
-            }
-          else
-            {
-              poke_mi_p = 1;
-            }
-          break;
         case 'L':
           poke_interactive_p = 0;
           return;
@@ -404,7 +388,25 @@ parse_args_1 (int argc, char *argv[])
         case NO_INIT_FILE_ARG:
           poke_load_init_file = 0;
           break;
+        case '?':
+          exit (EXIT_FAILURE);
+          break;
+        case QUIET_ARG:
+        case LOAD_ARG:
+        case LOAD_AND_EXIT_ARG:
+        case CMD_ARG:
+        case SOURCE_ARG:
+        case COLOR_ARG:
+        case STYLE_ARG:
+        case STYLE_BRIGHT_ARG:
+        case STYLE_DARK_ARG:
+        case 'c':
+        case 'l':
+        case 's':
+          /* Handled in parse_args_2.  */
+          break;
         default:
+          assert (!"Did you forget to add a CASE for a flag in parse_args_1?");
           break;
         }
     }
@@ -419,7 +421,7 @@ parse_args_2 (int argc, char *argv[])
   optind = 1;
   while ((ret = getopt_long (argc,
                              argv,
-                             "ql:c:s:L:",
+                             poke_getopt_string,
                              long_options,
                              NULL)) != -1)
     {
@@ -483,13 +485,19 @@ parse_args_2 (int argc, char *argv[])
             exit (exit_status);
             break;
           }
-        case MI_ARG:
         case NO_AUTO_MAP_ARG:
         case NO_HSERVER_ARG:
         case 'q':
         case NO_INIT_FILE_ARG:
           /* These are handled in parse_args_1.  */
           break;
+        case HELP_ARG:
+        case VERSION_ARG:
+          /* Handled in parse_args_1.  */
+          break;
+        case LOAD_AND_EXIT_ARG:
+          /* Handled in set_script_args.  */
+          break;
           /* libtextstyle arguments are handled in pk-term.c, not
              here.   */
         case COLOR_ARG:
@@ -497,8 +505,11 @@ parse_args_2 (int argc, char *argv[])
         case STYLE_DARK_ARG:
         case STYLE_BRIGHT_ARG:
           break;
-        default:
+        case '?':
           goto exit_failure;
+        default:
+          assert (!"Did you forget to add a case for a flag in parse_args_2?");
+          break;
         }
     }
 
@@ -551,12 +562,16 @@ initialize (int argc, char *argv[])
   if (poke_datadir == NULL)
     poke_datadir = PKGDATADIR;
 
+  poke_appdir = getenv ("POKEAPPDIR");
+  if (poke_appdir == NULL)
+    {
+      poke_appdir = pk_str_concat (poke_datadir, "/poke", NULL);
+      pk_assert_alloc (poke_appdir);
+    }
+
   poke_picklesdir = getenv ("POKEPICKLESDIR");
   if (poke_picklesdir == NULL)
-    {
-      poke_picklesdir = pk_str_concat (poke_datadir, "/pickles", NULL);
-      pk_assert_alloc (poke_picklesdir);
-    }
+    poke_picklesdir = "%DATADIR%/pickles";
 
   poke_mapsdir = getenv ("POKEMAPSDIR");
   if (poke_mapsdir == NULL)
@@ -567,7 +582,7 @@ initialize (int argc, char *argv[])
 
   poke_docdir = getenv ("POKEDOCDIR");
   if (poke_docdir == NULL)
-    poke_docdir = poke_datadir;
+    poke_docdir = poke_appdir;
 
   poke_infodir = getenv ("POKEINFODIR");
   if (poke_infodir == NULL)
@@ -586,6 +601,26 @@ initialize (int argc, char *argv[])
                  pk_make_int (poke_interactive_p, 32)) == PK_ERROR)
     pk_fatal ("defining poke_interactive_p");
 
+  /* Add the directories where poke installs .pk files to the
+     compiler's load path.  */
+  {
+    pk_val load_path = pk_decl_val (poke_compiler, "load_path");
+    char *newpaths;
+
+    assert (load_path != PK_NULL);
+    newpaths = pk_str_concat (pk_string_str (load_path),
+                              ":",
+                              poke_appdir,
+                              ":",
+                              poke_picklesdir,
+                              NULL);
+    pk_assert_alloc (newpaths);
+
+    pk_decl_set_val (poke_compiler,
+                     "load_path", pk_make_string (newpaths));
+    free (newpaths);
+  }
+
   /* The Poke hyperlinks facilities must be loaded before poke.pk and
      the cmd subsystem.  This is done even if the hserver is
      disabled.  */
@@ -652,7 +687,6 @@ initialize (int argc, char *argv[])
 #ifdef HAVE_HSERVER
   poke_hserver_p = (poke_interactive_p
                     && pk_term_color_p ()
-                    && !poke_mi_p
                     && !poke_no_hserver_arg);
 
   /* Initialize and start the terminal hyperlinks server.  */
@@ -702,16 +736,31 @@ initialize_user (void)
      use all the : separated paths in that variable to find
      pokerc.conf.  Else, try to load /etc/xdg/poke/pokerc.conf.  */
   {
-    const char *xdg_config_home = getenv ("XDG_CONFIG_HOME");
+    char *xdg_config_home = getenv ("XDG_CONFIG_HOME");
     const char *xdg_config_dirs = getenv ("XDG_CONFIG_DIRS");
 
     if (xdg_config_home == NULL)
-      xdg_config_home = "";
+      {
+        /* If unset, the default value of $HOME/.config should be preferred to
+           the paths in $XDG_CONFIG_DIRS. From the standard:
+
+           A user-specific version of the configuration file may be created in
+           $XDG_CONFIG_HOME/subdir/filename, taking into account the default
+           value for $XDG_CONFIG_HOME if $XDG_CONFIG_HOME is not set. */
+        xdg_config_home = pk_str_concat (getenv("HOME"), "/.config", NULL);
+        pk_assert_alloc (xdg_config_home);
+      }
+    else
+      {
+        /* strdup() the user-provided value, to free it below. */
+        xdg_config_home = strdup(xdg_config_home);
+        pk_assert_alloc (xdg_config_home);
+      }
 
     if (xdg_config_dirs == NULL)
       xdg_config_dirs = "/etc/xdg";
 
-    char *config_path = pk_str_concat (xdg_config_dirs, ":", xdg_config_home, NULL);
+    char *config_path = pk_str_concat (xdg_config_home, ":", xdg_config_dirs, NULL);
     pk_assert_alloc (config_path);
 
     char *dir = strtok (config_path, ":");
@@ -740,6 +789,7 @@ initialize_user (void)
     while ((dir = strtok (NULL, ":")) != NULL);
 
     free (config_path);
+    free (xdg_config_home);
   }
 }
 
@@ -749,7 +799,8 @@ poke_handle_exception (pk_val exception)
   pk_val handler = pk_decl_val (poke_compiler, "pk_exception_handler");
 
   assert (handler != PK_NULL);
-  if (pk_call (poke_compiler, handler, NULL, 1, exception) == PK_ERROR)
+  if (pk_call (poke_compiler, handler, NULL, NULL, 1, exception)
+      == PK_ERROR)
     assert (0);
 }
 
@@ -784,13 +835,8 @@ main (int argc, char *argv[])
      for IO, etc etc */
   parse_args_2 (argc, argv);
 
-  /* Enter the REPL or MI.  */
-  if (poke_mi_p)
-    {
-      if (!pk_mi ())
-        poke_exit_code = EXIT_FAILURE;
-    }
-  else if (poke_interactive_p)
+  /* Enter the REPL.  */
+  if (poke_interactive_p)
     pk_repl ();
 
   /* Cleanup.  */
